from typing import List, Optional

from nonebot import get_driver
from pydantic import BaseSettings, root_validator


class Config(BaseSettings):
    onebot_monitor_db_conn_url: str = "sqlite+aiosqlite:///onebot_monitor.db"
    onebot_monitor_ignore: List[str] = []

    onebot_monitor_auto_approve_friend_add_request: bool = False
    onebot_monitor_auto_approve_group_invite_request: bool = False

    onebot_monitor_forward_to: Optional[int] = None
    onebot_monitor_forward_request: bool = False
    onebot_monitor_forward_notice: bool = False

    @root_validator(allow_reuse=True)
    def validate_forward(cls, values):
        if not values["onebot_monitor_forward_to"]:
            if values["onebot_monitor_forward_request"]:
                raise ValueError("onebot_monitor_forward_to is None but onebot_monitor_forward_request is True")
            if values["onebot_monitor_forward_notice"]:
                raise ValueError("onebot_monitor_forward_to is None but onebot_monitor_forward_notice is True")
        return values

    class Config:
        extra = "ignore"


conf = Config(**get_driver().config.dict())

__all__ = ("conf", "Config")



from typing import Optional

from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageEvent
from nonebot.internal.matcher import current_bot


def get_reply_message_id(event: MessageEvent) -> Optional[int]:
    message_id = None
    for seg in event.original_message:
        if seg.type == "reply":
            message_id = int(seg.data["id"])
            break
    return message_id


async def map_group(group_id: int):
    bot = current_bot.get()
    try:
        group_info = await bot.get_group_info(group_id=group_id)
        return f"{group_info['group_name']} ({group_id})"
    except Exception as e:
        logger.exception(e)
        return str(group_id)


async def map_user(user_id: int):
    bot = current_bot.get()
    try:
        user_info = await bot.get_stranger_info(user_id=user_id)
        return f"{user_info['nickname']} ({user_id})"
    except Exception as e:
        logger.exception(e)
        return str(user_id)



from enum import Enum
from enum import Enum
from typing import Optional

from sqlalchemy import Column, Integer, BigInteger, JSON, String, Enum as SqlEnum

from nonebot_plugin_onebot_monitor.models import data_source


class NoticeType(str, Enum):
    group_upload = "group_upload"
    group_admin = "group_admin"
    group_decrease = "group_decrease"
    group_increase = "group_increase"
    group_ban = "group_ban"
    friend_add = "friend_add"
    group_recall = "group_recall"
    friend_recall = "friend_recall"
    group_card = "group_card"
    offline_file = "offline_file"
    client_status = "client_status"
    essence = "essence"
    notify = "notify"


@data_source.registry.mapped
class NoticeOrm:
    __tablename__ = "notices"

    id: int = Column(Integer, primary_key=True, autoincrement=True)
    time: int = Column(BigInteger, nullable=False)
    self_id: int = Column(BigInteger, nullable=False)

    notice_type: NoticeType = Column(SqlEnum(NoticeType), nullable=False)
    sub_type: Optional[str] = Column(String)

    user_id: Optional[int] = Column(BigInteger)
    group_id: Optional[int] = Column(BigInteger)

    extra: dict = Column(JSON)



from nonebot import on_notice, logger
from nonebot.adapters.onebot.v11 import NoticeEvent, Bot, Message, MessageSegment, GroupAdminNoticeEvent, \
    GroupDecreaseNoticeEvent, GroupIncreaseNoticeEvent, GroupBanNoticeEvent, GroupRecallNoticeEvent, PokeNotifyEvent, \
    LuckyKingNotifyEvent, HonorNotifyEvent, FriendAddNoticeEvent

from nonebot_plugin_onebot_monitor.config import conf
from nonebot_plugin_onebot_monitor.models import data_source
from nonebot_plugin_onebot_monitor.models.notice import NoticeOrm
from nonebot_plugin_onebot_monitor.utils import map_group, map_user


def parse_notice(event: NoticeEvent) -> NoticeOrm:
    event_dict = event.dict()

    del event_dict["post_type"]

    ntc = NoticeOrm()
    for col_name in NoticeOrm.__dict__:
        if col_name in event_dict:
            setattr(ntc, col_name, event_dict[col_name])
            del event_dict[col_name]
    ntc.extra = event_dict

    return ntc


async def map_notice_forward_message(notice: NoticeEvent):
    msg = None
    if isinstance(notice, GroupAdminNoticeEvent):
        if notice.sub_type == 'set':
            msg = f"Bot被设置为群聊{await map_group(notice.group_id)}的管理员"
        else:
            msg = f"Bot被取消群聊{await map_group(notice.group_id)}的管理员"
    elif isinstance(notice, GroupDecreaseNoticeEvent):
        if notice.sub_type == 'leave':
            msg = f"Bot主动退出群聊{await map_group(notice.group_id)}"
        else:
            msg = f"Bot被踢出群聊{await map_group(notice.group_id)}（操作人：{await map_user(notice.operator_id)}）"
    elif isinstance(notice, GroupIncreaseNoticeEvent):
        msg = f"Bot加入群聊{await map_group(notice.group_id)}"
    elif isinstance(notice, GroupBanNoticeEvent):
        if notice.sub_type == 'ban':
            msg = f"Bot在群聊{await map_group(notice.group_id)}被禁言{notice.duration / 60}分钟（操作人：{await map_user(notice.operator_id)}）"
        else:
            msg = f"Bot在群聊{await map_group(notice.group_id)}被解除禁言"
    elif isinstance(notice, GroupRecallNoticeEvent):
        if notice.operator_id == notice.self_id:
            msg = f"Bot在群聊{await map_group(notice.group_id)}主动撤回一条消息"
        else:
            msg = f"Bot在群聊{await map_group(notice.group_id)}被撤回一条消息（操作人：{await map_user(notice.operator_id)}）"
    elif isinstance(notice, FriendAddNoticeEvent):
        msg = f"Bot添加了新的好友{await map_user(notice.user_id)}"
    elif isinstance(notice, PokeNotifyEvent):
        if notice.group_id is not None:
            msg = f"Bot在群聊{await map_group(notice.group_id)}被戳一戳（操作人：{await map_user(notice.user_id)}）"
        else:
            msg = f"Bot在私聊中被{await map_user(notice.user_id)}戳一戳"
    elif isinstance(notice, LuckyKingNotifyEvent):
        msg = f"Bot在群聊{await map_group(notice.group_id)}成为红包运气王"
    elif isinstance(notice, HonorNotifyEvent):
        if notice.honor_type == 'talkative':
            msg = f"Bot在群聊{await map_group(notice.group_id)}成为龙王"
        elif notice.honor_type == 'performer':
            msg = f"Bot在群聊{await map_group(notice.group_id)}获得群聊之火"
        elif notice.honor_type == 'emotion':
            msg = f"Bot在群聊{await map_group(notice.group_id)}成为群聊之火"

    if msg is not None:
        return Message(MessageSegment.text(msg))
    else:
        return None


def _notice(event: NoticeEvent):
    # FriendAddNoticeEvent单独判断
    return ((event.is_tome() or isinstance(event, FriendAddNoticeEvent))
            and event.get_event_name() not in conf.onebot_monitor_ignore)


monitor_notice_matcher = on_notice(rule=_notice, priority=1)


@monitor_notice_matcher.handle()
async def record_notice(event: NoticeEvent):
    session = data_source.session()
    notice = parse_notice(event)
    session.add(notice)
    await session.commit()

    logger.success(f"recorded notice {notice}")


if conf.onebot_monitor_forward_notice:
    @monitor_notice_matcher.handle()
    async def forward_notice(bot: Bot, event: NoticeEvent):
        msg = await map_notice_forward_message(event)
        await bot.send_private_msg(user_id=conf.onebot_monitor_forward_to, message=msg)
        logger.success(f"forwarded notice {event}")



from enum import Enum
from typing import Optional

from sqlalchemy import Column, BigInteger, Integer, String, Enum as SqlEnum, JSON

from nonebot_plugin_onebot_monitor.models import data_source


class RequestType(str, Enum):
    friend = "friend"
    group = "group"


@data_source.registry.mapped
class RequestOrm:
    __tablename__ = "requests"

    id: int = Column(Integer, primary_key=True, autoincrement=True)
    time: int = Column(BigInteger, nullable=False)
    self_id: int = Column(BigInteger, nullable=False)

    request_type: RequestType = Column(SqlEnum(RequestType), nullable=False)
    sub_type: Optional[str] = Column(String)

    user_id: int = Column(BigInteger, nullable=False)
    group_id: Optional[int] = Column(BigInteger)

    extra: dict = Column(JSON)



from cachetools import TTLCache
from nonebot import on_request, logger, on_fullmatch
from nonebot.adapters.onebot.v11 import Bot, GroupRequestEvent, Event, Message, MessageSegment, PrivateMessageEvent, \
    FriendRequestEvent, ActionFailed
from nonebot.internal.matcher import Matcher

from .config import conf
from .utils import get_reply_message_id, map_user, map_group

context = TTLCache(4096, 86400 * 3)
latest_request = TTLCache(4096, 86400 * 3)


# ========== Friend Add ==========
def _friend_add_request(event: Event) -> bool:
    return isinstance(event, FriendRequestEvent)


friend_add = on_request(_friend_add_request)

if conf.onebot_monitor_auto_approve_friend_add_request:
    @friend_add.handle()
    async def auto_approve_friend_add(bot: Bot, event: FriendRequestEvent, matcher: Matcher):
        await event.approve(bot)
        logger.success(f"auto approved {event}")
        await matcher.finish()

if conf.onebot_monitor_forward_request:
    @friend_add.handle()
    async def forward_friend_add(bot: Bot, event: FriendRequestEvent):
        msg = Message(MessageSegment.text(f"收到来自用户{await map_user(event.user_id)}的好友申请。回复“同意”或“拒绝”处理此申请。"))
        send_result = await bot.send_private_msg(user_id=conf.onebot_monitor_forward_to, message=msg)
        logger.success(f"forwarded request {event}")

        context[send_result["message_id"]] = event
        latest_request[bot.self_id] = event


# ========== Group Invite ==========
def _group_invite_request(event: Event) -> bool:
    return isinstance(event, GroupRequestEvent) and event.sub_type == "invite"


group_invite = on_request(_group_invite_request)

if conf.onebot_monitor_auto_approve_group_invite_request:
    @group_invite.handle()
    async def auto_approve_group_invite(bot: Bot, event: GroupRequestEvent, matcher: Matcher):
        await event.approve(bot)
        logger.success(f"auto approved {event}")
        await matcher.finish()

if conf.onebot_monitor_forward_request:
    @group_invite.handle()
    async def forward_group_invite(bot: Bot, event: GroupRequestEvent):
        msg = Message(MessageSegment.text(f"收到来自群{await map_group(event.group_id)}的入群邀请。回复“同意”或“拒绝”处理此申请。"))
        send_result = await bot.send_private_msg(user_id=conf.onebot_monitor_forward_to, message=msg)
        logger.success(f"forwarded request {event}")

        context[send_result["message_id"]] = event
        latest_request[bot.self_id] = event

# ========== Operation ==========
if conf.onebot_monitor_forward_request:
    def make_operation_handler(for_approve: bool):
        async def handler(bot: Bot, event: PrivateMessageEvent, matcher: Matcher):
            request = None

            message_id = get_reply_message_id(event)
            if message_id is not None and message_id in context:
                request = context[message_id]
                del context[message_id]
                if bot.self_id in latest_request and latest_request[bot.self_id] == request:
                    del latest_request[bot.self_id]
            elif bot.self_id in latest_request:
                request = latest_request[bot.self_id]
                del latest_request[bot.self_id]

            if request is None:
                await matcher.finish("没有收到请求")

            try:
                if for_approve:
                    await request.approve(bot)
                    logger.success(f"approved {request}")
                    await matcher.send("成功同意请求")
                else:
                    await request.reject(bot)
                    logger.success(f"rejected {request}")
                    await matcher.send("成功拒绝请求")
            except ActionFailed as e:
                await matcher.send("请求处理失败，或许是因为你已经处理过该请求")
            except Exception as e:
                await matcher.send(f"内部错误：{type(e)}{str(e)}")

        return handler


    def _forward_to_only(event: PrivateMessageEvent):
        return event.user_id == conf.onebot_monitor_forward_to


    approve_matcher = on_fullmatch("同意", permission=_forward_to_only)
    approve_matcher.handle()(make_operation_handler(True))

    reject_matcher = on_fullmatch("拒绝", permission=_forward_to_only)
    reject_matcher.handle()(make_operation_handler(False))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nonebot_plugin_onebot_monitor', 'nonebot_plugin_onebot_monitor.models']

package_data = \
{'': ['*']}

install_requires = \
['aiosqlite>=0.17.0,<0.18.0',
 'cachetools>=5.2.0,<6.0.0',
 'nonebot-adapter-onebot>=2.1.3,<3.0.0',
 'nonebot-plugin-sqlalchemy>=0.1.0,<0.2.0',
 'nonebot2>=2.0.0-rc.1,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-onebot-monitor',
    'version': '0.1.2',
    'description': '',
    'long_description': 'None',
    'author': 'ssttkkl',
    'author_email': 'huang.wen.long@hotmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from nonebot import on_request, logger
from nonebot.adapters.onebot.v11 import RequestEvent

from nonebot_plugin_onebot_monitor.config import conf
from nonebot_plugin_onebot_monitor.models import data_source
from nonebot_plugin_onebot_monitor.models.request import RequestOrm


def parse_request(event: RequestEvent) -> RequestOrm:
    event_dict = event.dict()

    del event_dict["post_type"]

    ntc = RequestOrm()
    for col_name in RequestOrm.__dict__:
        if col_name in event_dict:
            setattr(ntc, col_name, event_dict[col_name])
            del event_dict[col_name]
    ntc.extra = event_dict

    return ntc


def _request(event: RequestEvent):
    return event.get_event_name() not in conf.onebot_monitor_ignore


monitor_request_matcher = on_request(rule=_request, priority=1)


@monitor_request_matcher.handle()
async def monitor_request(event: RequestEvent):
    session = data_source.session()
    request = parse_request(event)
    session.add(request)
    await session.commit()

    logger.success(f"recorded request {request}")



from nonebot import get_driver, require

from ..config import conf

require("nonebot_plugin_sqlalchemy")

from nonebot_plugin_sqlalchemy import DataSource

data_source = DataSource(get_driver(), conf.onebot_monitor_db_conn_url)

__all__ = ("data_source",)
